package SketchAggSensor.HammingSketch;

/**
 * <p>Title: Aggregation Query in Sensor Network with Sketches</p>
 * <p>Description: This code is for the simulation of sensor network with sketches algorithms. It applies with aggregation queries.</p>
 * <p>Copyright: Computer Science Department, Boston University,Copyright (c) 2003</p>
 * <p>Company: Boston University</p>
 * @author Feifei Li
 * @version 1.0
 */

import java.util.Random;

public class hammingSketch {

  public int numSketches;
  protected int bitmaps[];
  public double sketches[];
  public double p;
  public double scale;

  public hammingSketch(int _numSketches, double p, double scale) {
  this.numSketches = _numSketches;
  this.sketches = new double[this.numSketches];
  for (int i=0;i<this.numSketches;i++)
    this.sketches[i]=0.0;
  this.p = p;
  this.scale = scale;
  }

  //stable distribution
  double stable(double r1, double r2)
  {
        double theta = 3.141592654 * (r1-0.5);
        double f1 = Math.sin(p*theta) / Math.pow(Math.cos(theta), 1.0/p);
        double f2t = Math.cos(theta*(1.0-p));
        double f2b = -1.0 * Math.log(r2);
        return (f1*Math.pow(f2t/f2b, (1.0-p)/p));
  }

  // set the hamming norm sketches
  void insert(long _eid, double dk)
  {
        /*Random ran(_eid);  //the same set of random values will be produced if the seeds are the same
        double r1=ran.nextInt();
        double r2=ran.nextInt();*/
        Random ran = new Random(_eid);

        //printf("eid is : %d, r1 is %f, r2 is %f\n", _eid, r1, r2);
        for (int j=0; j<numSketches; j++)
        {
                //double r1 = (ran.nextInt()%10000)*0.0001;
                //double r2 = (ran.nextInt()%10000)*0.0001;
                double r1 = ran.nextDouble();
                double r2 = ran.nextDouble();
                double stable = this.stable(r1,r2);
                //System.out.println(r1+" "+r2+" "+stable);
                //double r1 = ran.nextDouble();
                //double r2 = ran.nextDouble();
                sketches[j] = sketches[j]+ dk*stable;
        }
  }

  double estimate()
  {
     for (int i=0;i<numSketches;i++)
       sketches[i] = Math.pow(Math.abs(sketches[i]),p);
     sort();
     //this.printSketches();
     //double mid = (sketches[numSketches/2-1] + sketches[numSketches/2])/2;
     double mid = sketches[numSketches/2];
     double est;
     est = mid/scale;
     return est;
  }

  void printSketches()
  {
    for (int i=0;i<this.numSketches;i++)
      System.out.println(sketches[i]);
  }


  void sort()
  {
        int i, j;
        double tmp = 0;
        for (i = 0; i < this.numSketches; i++)
        {
          for (j = i+1; j < this.numSketches; j++)
          {
               if (this.sketches[i] > this.sketches[j])
               {
                  tmp = this.sketches[i];
                  this.sketches[i] = this.sketches[j];
                  this.sketches[j] = tmp;
                }
          }
        }
  }


  public static void main(String[] args) {
    int iterations = 100;
    double arithmetic_total = 0.0;
    double geometric_total = 0.0;
    int numSketches = 200;
    double p=0.5;
    double scale = 1.1;

    double min = Double.POSITIVE_INFINITY;
    double max = Double.NEGATIVE_INFINITY;


    for(int i = 0; i < iterations; ++i)
    {
       hammingSketch sketch = new hammingSketch(numSketches,p,scale);
       Random random = new Random(i);
       long seed = random.nextLong();
       for(int j = 0; j < 5000; ++j)
       {
         sketch.insert(seed+j,1.0);
       }
       double estimate = sketch.estimate();
       arithmetic_total += estimate;
       geometric_total += Math.log(estimate);

       if(estimate < min)
         min = estimate;
       if(estimate > max)
         max = estimate;
       System.out.print(estimate);
       System.out.print("\n");
       //System.out.println("estimate = " + estimate);
    }

    System.out.println("min = " + min);
    System.out.println("max = " + max);

    System.out.println("arithmetic_mean = " + (arithmetic_total / iterations));
    System.out.println("geometric_mean = " + Math.exp(geometric_total / iterations));


  }
}