package SketchAggSensor.agg;

import SketchAggSensor.sensor.Sensor;
import SketchAggSensor.InvalidAggregateException;

/**
 * <p>Title: Aggregation Query in Sensor Network with Sketches</p>
 * <p>Description: This code is for the simulation of sensor network with sketches algorithms. It applies with aggregation queries.</p>
 * <p>Copyright: Computer Science Department, Boston University,Copyright (c) 2003</p>
 * <p>Company: Boston University</p>
 * @author Feifei Li
 * @version 1.0
 */

public class Count implements Aggregator {

  public Object combine(Sensor sensor, Object agg) throws InvalidAggregateException {
       int reading = sensor.getReading(sensor.QUERY_FIELD);
       if (! (agg instanceof CountAgg)) throw new InvalidAggregateException();
       ((CountAgg)agg).cnt += 1;
       ((CountAgg)agg).cnt = ((CountAgg)agg).cnt;
       return agg;
   }

   public Object combine(Object agg1, Object agg2) throws InvalidAggregateException {
       if (! (agg1 instanceof CountAgg)) throw new InvalidAggregateException();
       if (! (agg2 instanceof CountAgg)) throw new InvalidAggregateException();
       ((CountAgg)agg1).cnt += ((CountAgg)agg2).cnt;
       return agg1;
   }

   public Object emptyAgg(Sensor s) {
       return new CountAgg();
   }
   public int getValue(Object agg) throws InvalidAggregateException {
       if (! (agg instanceof CountAgg)) throw new InvalidAggregateException();
       return (int)Math.ceil(((CountAgg)agg).cnt);
   }

   public Object getValueObj(Object agg) throws InvalidAggregateException {
       return null;
   }

   public Object makeGuess() {
       CountAgg guess = new CountAgg();
       guess.cnt = 0;
       return guess;
   }
   /** Return the number of sensors contributing to this aggregate,
       or -1 if unknown */
   public int getCount(Object agg) throws InvalidAggregateException {
       if (!(agg instanceof CountAgg)) throw new InvalidAggregateException();
       return (int)Math.ceil(((CountAgg)agg).cnt);
   }

   public int numBytes(Object agg) throws InvalidAggregateException  {
       if (!(agg instanceof CountAgg)) throw new InvalidAggregateException();
       CountAgg ca = (CountAgg)agg;
       if (ca.divided)
         return FLOAT_BYTES;
       return INT_BYTES;
   }

   public void divide(Object agg, int divisor) throws InvalidAggregateException {
       if (!(agg instanceof CountAgg)) throw new InvalidAggregateException();
       CountAgg ca = (CountAgg)agg;
       ca.cnt = ca.cnt / (double)divisor;
       ca.divided = true;
   }

   public Object cloneAgg(Object agg) throws InvalidAggregateException {
       if (!(agg instanceof CountAgg)) throw new InvalidAggregateException();
       CountAgg ma = (CountAgg)agg, newAgg = (CountAgg)emptyAgg(null);
       newAgg.cnt = ma.cnt;
       return newAgg;
   }
   class CountAgg {
       public double cnt = 0;
       public boolean divided = false;
       public String toString() {
           String s = "" + cnt;
           return s;
    }
   }
}