package SketchAggSensor.agg;

import SketchAggSensor.sensor.Sensor;
import SketchAggSensor.InvalidAggregateException;
import SketchAggSensor.loglogsketch.*;
import java.util.*;

/**
 * <p>Title: Aggregation Query in Sensor Network with Sketches</p>
 * <p>Description: This code is for the simulation of sensor network with sketches algorithms. It applies with aggregation queries.</p>
 * <p>Copyright: Computer Science Department, Boston University,Copyright (c) 2003</p>
 * <p>Company: Boston University</p>
 * @author Feifei Li
 * @version 1.0
 */

public class CountLoglogAgg implements Aggregator {

  public Random random;

  public Object combine(Sensor sensor, Object agg) throws InvalidAggregateException {
     int reading = sensor.getReading(sensor.QUERY_FIELD);
     if (! (agg instanceof CountSketch)) throw new InvalidAggregateException();
     {
       random = new Random(sensor.queryID);
       int key =0;
       for (int i=0;i<sensor.getId();i++)
         key = random.nextInt();
       ((CountSketch)agg).insert(key);
       //((CountSketch)agg).insert(sensor.sketchInsertKey);
     }
     return agg;
 }

 public Object combine(Object agg1, Object agg2) throws InvalidAggregateException {
     if (! (agg1 instanceof CountSketch)) throw new InvalidAggregateException();
     if (! (agg2 instanceof CountSketch)) throw new InvalidAggregateException();
     CountSketch ma1 = (CountSketch)agg1;
     CountSketch ma2 = (CountSketch)agg2;
     CountSketch newAgg = new CountSketch(Sketch.mergeSketch(ma1,ma2));
     return newAgg;
 }

 public Object emptyAgg(Sensor s) {
     CountSketch agg =  new CountSketch(Sensor.kvalLoglog);
     return agg;
 }
 public int getValue(Object agg) throws InvalidAggregateException {
     if (! (agg instanceof CountSketch)) throw new InvalidAggregateException();
    return (int)((CountSketch)agg).estimate();
 }

 public Object getValueObj(Object agg) throws InvalidAggregateException {
     return null;
 }

 public Object makeGuess() {
    System.out.println("in guess\n");
    CountSketch guess = new CountSketch(3);
     return guess;
 }
 /** Return the number of sensors contributing to this aggregate,
     or -1 if unknown */
 public int getCount(Object agg) throws InvalidAggregateException {
     if (!(agg instanceof CountSketch)) throw new InvalidAggregateException();
     int a= (int)((CountSketch)agg).estimate();
     //System.out.println("in sketchAgg getCount:"+a+"\n");
     return a;
 }

 public int numBytes(Object agg) throws InvalidAggregateException  {
     if (!(agg instanceof CountSketch)) throw new InvalidAggregateException();
     //return INT_BYTES;
     CountSketch s = (CountSketch)agg;
     int totalBits =0;
   /*
     if (Sensor.COMPRESSION_GROUP == Sensor.numBitMaps)
     {
       if (Sensor.COMPRESSION_SCHEME == 1) //our compression algo
       {  s.SetKey();
          totalBits = (s.bitMapLength-s.key0-s.key1)*s.numBitMap;
          totalBits +=4;
       }
       else if (Sensor.COMPRESSION_SCHEME == 2) //ANF compression algo
       {
         s.compressionANF(0,s.numBitMap);
         totalBits = s.bitMapLength*s.numBitMap-s.key0-s.key1+4;
       }
     }
     else
     {
       int numGroup = Sensor.numBitMaps / Sensor.COMPRESSION_GROUP;
       for (int k= 1; k<=numGroup; k++)
       {
         int start = Sensor.COMPRESSION_GROUP*(k-1);
         int stop = Sensor.COMPRESSION_GROUP*k;
         if (Sensor.COMPRESSION_SCHEME == 1)
         {  s.SetKey(start,stop);
            totalBits += (s.bitMapLength-s.key0-s.key1)*Sensor.COMPRESSION_GROUP;
            totalBits +=4;
         }
         else if (Sensor.COMPRESSION_SCHEME ==2 )
         {
            s.compressionANF(start,stop);
            totalBits += s.bitMapLength*Sensor.COMPRESSION_GROUP-s.key0-s.key1;
            totalBits += 4;
         }
       }
     }
   */
     totalBits+=s.numBitMap*4;
     return totalBits;
 }


   public Object cloneAgg(Object agg) throws InvalidAggregateException {
     if (!(agg instanceof CountSketch)) throw new InvalidAggregateException();
     CountSketch ma = (CountSketch)agg;
     CountSketch newAgg = (CountSketch)Sketch.cloneSketch(ma);

     return newAgg;
   }

   public void divide(Object agg, int divisor) throws InvalidAggregateException {
     /*   if (!(agg instanceof CountAgg)) throw new InvalidAggregateException();
     CountAgg ca = (CountAgg)agg;
     ca.cnt = ca.cnt / (double)divisor;
     */
 }


}
