package SketchAggSensor.agg;

import SketchAggSensor.sensor.Sensor;
import SketchAggSensor.InvalidAggregateException;
import java.util.Random;
/**
 * <p>Title: Aggregation Query in Sensor Network with Sketches</p>
 * <p>Description: This code is for the simulation of sensor network with sketches algorithms. It applies with aggregation queries.</p>
 * <p>Copyright: Computer Science Department, Boston University,Copyright (c) 2003</p>
 * <p>Company: Boston University</p>
 * @author Feifei Li
 * @version 1.0
 */

public class Sum implements Aggregator {

  public Object combine(Sensor sensor, Object agg) throws InvalidAggregateException {
    int reading = sensor.getReading(sensor.QUERY_FIELD);
    if (!(agg instanceof SumAgg)) throw new InvalidAggregateException();
    SumAgg s = (SumAgg) agg;
    s.sum += reading;
    s.cnt++;
    return agg;
  }

  public Object combine(Object agg1, Object agg2) throws InvalidAggregateException {
    if (!(agg1 instanceof SumAgg)) throw new InvalidAggregateException();
    if (!(agg2 instanceof SumAgg)) throw new InvalidAggregateException();

    SumAgg s1 = (SumAgg)agg1;
    SumAgg s2 = (SumAgg)agg2;
    s1.sum +=s2.sum;
    s1.cnt +=s2.cnt;
    return s1;
  }

  public Object emptyAgg(Sensor s) {
    return new SumAgg();
  }

  public int getValue(Object agg) throws InvalidAggregateException {
    if (!(agg instanceof SumAgg)) throw new InvalidAggregateException();
    return (int)((SumAgg)agg).sum;
  }

  public Object getValueObj(Object agg) throws InvalidAggregateException {
    return null;
  }

  public int getCount(Object agg) throws InvalidAggregateException {
    return ((SumAgg)agg).cnt;
  }

  public int numBytes(Object agg) throws InvalidAggregateException {
    if (!(agg instanceof SumAgg)) throw new InvalidAggregateException();
    return INT_BYTES;
  }

  public void divide(Object agg, int divisor) throws InvalidAggregateException {
    if (!(agg instanceof SumAgg)) throw new InvalidAggregateException();
      SumAgg s = (SumAgg)agg;
      s.sum = s.sum / (double)divisor;
  }


  public Object cloneAgg(Object agg) throws InvalidAggregateException {
    if (!(agg instanceof SumAgg)) throw new InvalidAggregateException();
    SumAgg s = (SumAgg)agg;
    SumAgg newAgg = (SumAgg)emptyAgg(null);
    newAgg.sum = s.sum;
    newAgg.cnt = s.cnt;
    return newAgg;
  }

  public Object makeGuess() {
    SumAgg guess = new SumAgg();
    guess.sum = 99;
    guess.cnt = 1;
    return guess;
  }

  class SumAgg {
    public double sum = 0;
    public int cnt = 0;

    public String toString() {
      String s ="" + sum;
      return s;
    }
    }
  }