package SketchAggSensor.loglogsketch;

/**
 * <p>Title: Aggregation Query in Sensor Network with Sketches</p>
 * <p>Description: This code is for the simulation of sensor network with sketches algorithms. It applies with aggregation queries.</p>
 * <p>Copyright: Computer Science Department, Boston University,Copyright (c) 2003</p>
 * <p>Company: Boston University</p>
 * @author Feifei Li
 * @version 1.0
 */

import java.util.Random;

public class CountSketch extends Sketch
{
  public CountSketch()
  {
    super();
  }

  public CountSketch(int kIn)
  {
    super(kIn);
  }

  public CountSketch (Sketch sketch)
  {
    super(sketch.k);
    for (int i=0;i<sketch.numBitMap;i++)
        this.counterM[i] = sketch.counterM[i];
  }

  public synchronized void insert(long keyIn)
  {

    Random tempRandom = new Random(keyIn);

    //int bitmapChoice = tempRandom.nextInt(this.numBitMap);
     int x = 0;
    for (int i=0;i<this.bitMapLength;i++)
    {  bitmaps[i]=0;
       if (tempRandom.nextBoolean())
         bitmaps[i]=1;
       //System.out.print(bitmaps[i]);
    }
    //System.out.println("");
    int valJ=0;
    for (int i=0;i<this.k;i++)
    {
      if (bitmaps[i]==1)
        valJ+=(int)Math.pow(2.0,(double)(k-1-i));
    }
    //System.out.println("valJ:"+valJ);
    int pos=k;
    while(pos<this.bitMapLength  && bitmaps[pos]==0)
      pos++;
    int rho = pos-k;
    //System.out.println("rho:"+rho);
    if (pos<this.bitMapLength)
    {   if (this.counterM[valJ]<rho)
           this.counterM[valJ] = rho;
    }
    else
      this.counterM[valJ] = this.bitMapLength-k;
   }

  public static void main(String[] args)
  {
    if((args == null) || (args.length != 1))
    {
        usage();
    }

    int k = -1;
    try
    {
        k = new Integer(args[0]).intValue();
    }
    catch(NumberFormatException e)
    {
        usage();
    }

    if(k <= 0)
    {
        usage();
    }

    int iterations = 100;
    double arithmetic_total = 0.0;
    double geometric_total = 0.0;

    double entropy_total = 0.0;

    double min = Double.POSITIVE_INFINITY;
    double max = Double.NEGATIVE_INFINITY;


    for(int i = 0; i < iterations; ++i)
    {
       CountSketch sketch = new CountSketch(k);
       Random random = new Random(i);

        for(int j = 0; j < 5000; ++j)
        {
            sketch.insert(random.nextInt());
            //System.out.println("inserting "+(j+1));
            //sketch.printSketch();
           // sketch.compression();
            //System.out.print("\n");
          }
       /* random = new Random(i);

          for(int j = 0; j < 5000; ++j)
       {
           sketch.insert(random.nextInt());
           //System.out.println("inserting "+(j+1));
           //sketch.printSketch();
           //System.out.print("\n");
         }
        */

        double estimate = sketch.estimate();
        arithmetic_total += estimate;
        geometric_total += Math.log(estimate);

        if(estimate < min)
          min = estimate;
        if(estimate > max)
          max = estimate;
        double entropy = sketch.entropy();
        entropy_total+=entropy;
        System.out.print(estimate+"  "+entropy);
        System.out.print("\n");
           //sketch.compression();
        //System.out.println("estimate = " + estimate);
      }

    System.out.println("min = " + min);
    System.out.println("max = " + max);

    System.out.println("arithmetic_mean = " + (arithmetic_total / iterations));
    System.out.println("geometric_mean = " + Math.exp(geometric_total / iterations));

    System.out.println("entropy_mean = " + (entropy_total / iterations));
  }

  static void usage()
  {
    System.err.println("java CountSketch <k>");
    System.exit(1);
  }

}
