package SketchAggSensor.message;

/**
 * <p>Title: Aggregation Query in Sensor Network with Sketches</p>
 * <p>Description: This code is for the simulation of sensor network with sketches algorithms. It applies with aggregation queries.</p>
 * <p>Copyright: Computer Science Department, Boston University,Copyright (c) 2003</p>
 * <p>Company: Boston University</p>
 * @author Feifei Li
 * @version 1.0
 */

import SketchAggSensor.*;
import SketchAggSensor.sensor.*;
import SketchAggSensor.parent.*;


import java.util.*;

public class AggMessage extends Message {

  static final int NO_GROUP = 0;
   static final int TEMP_SENSOR = 1;
   static final int LIGHT_SENSOR = 2;

   /** The guess (hypothesis) aggregate asssociated with this message, if any. */
      public Object guessAgg = null;
      /** The groupBy predicate associated with this message */
      public Grouping groupBy;
      /** Having predicate, currently unsupported. */
      public Predicate having;

      public int level;
      public Vector parents;

      Vector aggregates;
      Vector aggregates1;
   /** Constructor
       @param sender The sensor that is sending the message
       @param epoch The epoch in which the message was sent (see {@link Clock#getEpoch(int) Clock.getEpoch})
       @param aggs A vector of partial state records representing the aggregates
                   from this child and its children partitioned by group.
       @param level The level of the sender
       @param parent The parent of the sender
       @param groupBy The grouping applied to aggs
       @param having The having predicate (not currently supported)
       @param bytes The number of bytes required for this message (hack used for state measurement)
   */
   public AggMessage(Sensor sender, int epoch, Vector aggs, int level,
                     Vector parents, Grouping groupBy, Predicate having, int bytes) {
       super(sender,epoch, bytes,false);

       this.level = level;
       this.groupBy = groupBy;
       this.having = having;
       this.aggregates = aggs;
       this.parents = parents;
   }

   /** Constructor for non-grouped aggregates
       @param sender The sensor that is sending the message
       @param epoch The epoch in which the message was sent (see {@link Clock#getEpoch(int) Clock.getEpoch()} )
       @param agg A partial state records representing the aggregates
                   from this child and its children.
       @param level The level of the sender
       @param parent The parent of the sender
       @param bytes The number of bytes required for this message (hack used for state measurement)
   */
   public AggMessage(Sensor sender, int epoch, Object agg,
                     int level, Vector parents, int bytes) {
       super(sender, epoch, bytes,false);

       Vector aggs = new Vector();
       aggs.addElement(agg);
       this.level = level;
       this.groupBy = null;
       this.having = null;
       this.aggregates = aggs;
       this.parents = parents;
   }

   public AggMessage(Sensor sender, int epoch, Object agg, Object agg1,
                  int level, Vector parents, int bytes) {
    super(sender, epoch, bytes,false);

    Vector aggs = new Vector();
    aggs.addElement(agg);
    Vector aggs1 = new Vector();
    aggs1.addElement(agg1);
    this.level = level;
    this.groupBy = null;
    this.having = null;
    this.aggregates = aggs;
    this.aggregates1 = aggs1;
    this.parents = parents;
}


   /** @return the value of the aggregate in this message. Value can be decoded by calling
       {@link Aggregator#getValue(Object) Aggregator.getValue()}
       If the aggregate is grouped, return the value of the first group.
   */
   public Object getAggregate() {
       return aggregates.elementAt(0);
   }

   public Object getAggregate1() {
    return aggregates1.elementAt(0);
}


   /** @param group The group whose value should be fetched.
       @return The value of the specified group. Value can be decoded by calling
       {@link Aggregator#getValue(Object) Aggregator.getValue()}.
       @throws NoSuchElementException If the group is invalid.
   */
   public Object getAggregate(int group) throws NoSuchElementException{
       return aggregates.elementAt(group);
   }

   /** @return A reference to the vector of aggregates in this AggMessage */
   public Vector getAggregates() {
       return aggregates;
   }

   /** @return The level of the sender of this agg message */
   public int getLevel() {
       return level;
   }

   /** @return The parent of the sender of this agg message */
   public Vector getParent() {
       return parents;
   }

   /** @return A ParentInfo object representing the attributes of
       this sensor that can be used in parent selection
   */
   public ParentInfo getInfo() {
       return new ParentInfo(getLevel(), sender.getReading(0), getParent());
   }

   public String toString() {
       String s = "Count : " + aggregates.elementAt(0) + " , level = " + level;
       return s;

   }

}