package SketchAggSensor.network;

/**
 * <p>Title: Aggregation Query in Sensor Network with Sketches</p>
 * <p>Description: This code is for the simulation of sensor network with sketches algorithms. It applies with aggregation queries.</p>
 * <p>Copyright: Computer Science Department, Boston University,Copyright (c) 2003</p>
 * <p>Company: Boston University</p>
 * @author Feifei Li
 * @version 1.0
 */
import java.util.*;
import SketchAggSensor.sensor.*;
import SketchAggSensor.topology.*;
import SketchAggSensor.message.*;
import SketchAggSensor.agg.*;

import SketchAggSensor.*;


public class SensorNetwork {

  static  int NUM_SENSORS = 900;
  static  int GRID_SIZE = 30;
  static  int COMM_RADIUS = 1;
  static  double LOSS_PROB = 0.05;
  static int COMPRESSION_GROUP = -1;
  static boolean RANDOM_PLACEMENT = false;
  static int dist = Sensor.RANDOM_DIST;
  static boolean USING_SKETCH = false;
  static boolean USING_LOGLOG = false;
  static boolean USING_ANF =false;
  static boolean MULTIPLE_PARENTS = true;
  static final int QUERY_SUM = 2;
  static final int QUERY_COUNT =1;
  static final int QUERY_COUNTLIST = 3;
  static final int QUERY_SUMLIST = 4;
  static int QUERY_TYPE = QUERY_COUNT;
  static boolean TWOAGG = false;
  static double loss[] = {0,0};
  static double real_loss[] = {0, .2475, .4, .56825, .9235, .98375};
  static double longlinks = .01;  //some small percentage links are long
  static boolean realistic = false;
  static int sendEvery = -1;
  Topology topology;

  public SensorNetwork()
  {
  }

  public SensorNetwork(Topology t) {
    this.topology = t;
  }

  public int simulate(int k)
  {
    Clock clock = new Clock();
    Topology t = new Topology(GRID_SIZE,GRID_SIZE,COMM_RADIUS);
    //NodeFailTopology t = new NodeFailTopology(GRID_SIZE,GRID_SIZE,COMM_RADIUS,LOSS_PROB);
    //LinkFailTopology t = new LinkFailTopology(GRID_SIZE,GRID_SIZE,COMM_RADIUS,0.1);
    int curid = 0;
    CountLoglogAgg cntloglogAgg = new CountLoglogAgg();
    CountSketchAgg cntsketchAgg = new CountSketchAgg();
    Count cnt = new Count();
    Sum sum = new Sum();
    SumSketchAgg sumsketchAgg = new SumSketchAgg();
    CountListAgg cntlist = new CountListAgg();
    SumListAgg sumlist = new SumListAgg();
    Aggregator agg = cnt;
    Aggregator agg1 = null;
    if (QUERY_TYPE == QUERY_COUNT)
    {  if (USING_SKETCH)
        agg = cntsketchAgg;
      else if (USING_LOGLOG)
        agg = cntloglogAgg;
      else
        agg = cnt;
      if (TWOAGG == true)
        agg1 = cntlist;
    }
    else if (QUERY_TYPE == QUERY_SUM)
    {
       if (USING_SKETCH)
         agg = sumsketchAgg;
       else
         agg = sum;
       if (TWOAGG == true)
         agg1 = sumlist;
    }
    if (QUERY_TYPE == QUERY_COUNTLIST)
    {  agg = cntlist;
       TWOAGG = false;
    }
    //k =1;
    if (QUERY_TYPE == QUERY_SUMLIST)
    {  agg = sumlist;
       TWOAGG = false;
    }
    NormalAggSensor s;
    AggInitiationSensor root=null;
    if (TWOAGG== false)
      root = new AggInitiationSensor(t,curid++,agg,dist);
    else
      root = new AggInitiationSensor(t,curid++,agg,agg1,dist);
    root.setQueryID(k);
    clock.addListener(root);
    t.addSensor(root, GRID_SIZE/2, GRID_SIZE/2);
    Random r = new Random();
    //Random r1 = new Random(k);
//    Enumeration ide = idVec.elements();
     for (int i = 0; i < NUM_SENSORS; i++) {
         int x,y;
         if (RANDOM_PLACEMENT) {
             x = r.nextInt(GRID_SIZE);
             y = r.nextInt(GRID_SIZE);

         } else {
             x = i % GRID_SIZE; //r.nextInt(GRID_SIZE);
             y = i / GRID_SIZE; // r.nextInt(GRID_SIZE);

         }
         if (TWOAGG == false)
           s = new NormalAggSensor(t,curid++,agg, dist);
         else
           s = new NormalAggSensor(t,curid++,agg, agg1, dist);
  //     Integer nextID = (Integer)ide.nextElement();
 //      s = new NormalAggSensor(t,nextID.intValue(),agg, dist);
         if (MULTIPLE_PARENTS)
           s.setMultipleParents(true);
         //s.setQueryID(k);
         clock.addListener(s);
         t.addSensor(s, x, y);
         //s.sketchInsertKey = r1.nextInt();
     }
     //make the topology last...
     clock.setFinalListener(t);
     this.topology=t;


     for (int i = 0; i < Sensor.MAX_EPOCHS+1; i++) {
         int tcnt = t.sentCount();
         clock.nextTime();
         clock.nextTime();
        // System.out.println("Round = " + i);
     }
     /*
     for (int i=0;i<= NUM_SENSORS;i++)
     {
       Sensor s1 = (Sensor)t.getSensorId(i);
       Vector par = s1.getParents();
       System.out.print("parents for sensor "+i+" at ("+s1.getX()+","+s1.getY()+") with level "+s1.getLevel());
       Enumeration e = par.elements();
       while(e.hasMoreElements())
       {
         Integer pid = (Integer)e.nextElement();
         System.out.print(" "+pid+" ");
       }
       System.out.println("");
     }*/
     //this.getStatistics(t);
     if (TWOAGG == false)
       System.out.print("  "+root.getAgg()+"\n");
     else
       System.out.print("  "+root.getAgg()+"  "+root.getAgg1()+"\n");
     return root.getAgg();
  }

  void getStatistics(Topology t)
  {
    int bytesSentTotal=0;
    int msgSentTotal=0;
    int msgReceivedTotal=0;
    Enumeration sensors = t.sensors();
    while (sensors.hasMoreElements())
    {
      Sensor s = (Sensor)sensors.nextElement();
      bytesSentTotal+=s.sentBytes();
      msgSentTotal+=s.sentCount();
      msgReceivedTotal+=s.msgCount();
    }
    //System.out.println("Total Bytes Sent = "+bytesSentTotal);
    //System.out.println("Total Messages Sent = "+msgSentTotal);
    //System.out.println("Total Messages Received = "+msgReceivedTotal);
    System.out.print(bytesSentTotal+" \n");
  }

  //final static String usage = "\n-size x | -sensors x | -radius x| -loss x| -sketch | -mp | -random | -normal | -9010 |-geo | -realistic | -changeevery\n";
  final static String usage = "\n -size x | -sensors x | -nodeloss x| -linkloss x |  -rounds x | -sum | -sketch | -mp | - divisor | -numbitmaps x | -bitmapslen x | -loglogkval x  | -countlist | -random | -sumlist | -twoagg | -compression x | -anf | -loglog\n";
  public static void main (String argv[]) {
     int ROUNDS = 30;
     //int ITERATIONS = 1000;
     for (int i = 0; i < argv.length; i++) {
           if (argv[i].equals("-size")) {
               GRID_SIZE = (new Integer(argv[++i])).intValue();
           } else
               if (argv[i].equals("-sensors")) {
                   NUM_SENSORS = (new Integer(argv[++i])).intValue();
           } else
               if (argv[i].equals("-radius")) {
                   COMM_RADIUS = (new Integer(argv[++i])).intValue();
           } else
               if (argv[i].equals("-nodeloss")) {
                   LOSS_PROB = (new Double(argv[++i])).doubleValue();
                   Sensor.NODE_FAILURE = LOSS_PROB;
           } else
               if (argv[i].equals("-numbitmaps")) {
                   Sensor.numBitMaps = (new Integer(argv[++i])).intValue();
           } else
               if (argv[i].equals("-loglogkval")) {
                   Sensor.kvalLoglog = (new Integer(argv[++i])).intValue();
           } else
               if (argv[i].equals("-bitmapslen")) {
                   Sensor.bitMapLength = (new Integer(argv[++i])).intValue();
           } else
               if (argv[i].equals("-linkloss")) {
                 LOSS_PROB = (new Double(argv[++i])).doubleValue();
                 Sensor.LINK_FAILURE = LOSS_PROB;
           } else
               if (argv[i].equals("-compression")) {
                 COMPRESSION_GROUP = (new Integer(argv[++i])).intValue();
           } else
               if (argv[i].equals("-rounds")) {
                  ROUNDS = (new Double(argv[++i])).intValue();
          } else
               if (argv[i].equals("-sketch")) {
                 USING_SKETCH = true;
          } else
             if (argv[i].equals("-loglog")) {
               USING_LOGLOG = true;
          } else
               if (argv[i].equals("-anf")) {
                 USING_ANF = true;
           } else
               if (argv[i].equals("-sum")) {
                 QUERY_TYPE = QUERY_SUM;
           } else
               if (argv[i].equals("-mp")) {
                 MULTIPLE_PARENTS = true;
           } else
               if (argv[i].equals("-divisor")) {
                 Sensor.USING_DIVISOR = true;
           } else
               if (argv[i].equals("-random")) {
                 RANDOM_PLACEMENT = true;
           } else
               if (argv[i].equals("-normal")) {
                   dist = Sensor.NORMAL_DIST;
           } else
               if (argv[i].equals("-countlist")) {
                   QUERY_TYPE =QUERY_COUNTLIST;
           } else
               if (argv[i].equals("-sumlist")) {
                   QUERY_TYPE =QUERY_SUMLIST;
          } else
               if (argv[i].equals("-twoagg")) {
                   TWOAGG = true;
           } else
               if (argv[i].equals("-9010")) {
                   dist = Sensor.NINETY_TEN_DIST;
           } else
               if (argv[i].equals("-geo")) {
                   dist = Sensor.GEOGRAPHIC_DIST;
           } else
               if (argv[i].equals("-realistic")) {
                   realistic = true;
           } else
               if (argv[i].equals("-changeevery")) {
                   sendEvery = (new Integer(argv[++i])).intValue();
           } else {
               System.out.println("Unknown parameter : " + argv[i]);
               System.out.println("USAGE : " + usage);
               System.exit(0);
           }
       }

       if (COMPRESSION_GROUP == -1)
         Sensor.COMPRESSION_GROUP = Sensor.numBitMaps;
       else
         Sensor.COMPRESSION_GROUP = COMPRESSION_GROUP;
       if (USING_ANF == true)
         Sensor.COMPRESSION_SCHEME =2;
        Sensor.MAX_EPOCHS = GRID_SIZE/2+1;

        //dist = Sensor.SUM_TEST_DIST;
        /*Random idr = new Random();
        Vector idVec = new Vector();
        int j=0;
        while (j<NUM_SENSORS)
        {
          Integer nextID = new Integer(idr.nextInt(10000));
          if (!idVec.isEmpty())
          {
            if (!idVec.contains(nextID))
            {  idVec.add(nextID);
               j++;
            }
          }
          else
          {  idVec.add(nextID);
             j++;
          }
        }
        */
        //Random r = new Random();
        int data[] = new int[ROUNDS];
        for (int i = 0; i < ROUNDS; i++) {
          SensorNetwork v = new SensorNetwork();
          //int d = r.nextInt(ITERATIONS);
          int value;

         do
          { value = v.simulate(i);
          }
          while (value==0);
            //System.out.println(i+" Round\tValue");
            //System.out.println("---------------------");
            //System.out.print(value+"\n");
            data[i] = value;
        }

        double arithmetic_total = 0.0;
        double geometric_total = 0.0;
        int errorRound =0;
        for (int i = 0 ; i < ROUNDS; i++) {
            //System.out.println(i + "," + data[i]);
            if (data[i]!=0)
            { arithmetic_total += data[i];
              geometric_total += Math.log(data[i]);
            }
            else
              errorRound++;
        }
        int totalRound = ROUNDS-errorRound;
        System.out.println("arithmetic_avg = " + (int)Math.ceil(arithmetic_total / totalRound));
        System.out.println("geometic_avg = " + (int)Math.ceil(Math.exp(geometric_total / totalRound)));
    }
}
