package SketchAggSensor.sensor;

/**
 * <p>Title: Aggregation Query in Sensor Network with Sketches</p>
 * <p>Description: This code is for the simulation of sensor network with sketches algorithms. It applies with aggregation queries.</p>
 * <p>Copyright: Computer Science Department, Boston University,Copyright (c) 2003</p>
 * <p>Company: Boston University</p>
 * @author Feifei Li
 * @version 1.0
 */

import SketchAggSensor.*;
import SketchAggSensor.message.*;
import SketchAggSensor.topology.*;
import SketchAggSensor.agg.*;
import java.util.*;

public class AggInitiationSensor extends NormalAggSensor implements ClockListener{


    boolean once = false;
    boolean guess = false;


    /** Constructor
        @param t -- the topology where this sensor lives
        @param id The id of this sensor
        @param agg The aggregation function running on this sensors
        @param dist The distribution of sensor values for this sensor
        @param parentReading unused
        @param group Should we enable grouping ?
        @param guess Should we enable hypothesis testing ?
        @param softState Should we cache child results ?
        @param softStateRecall The number epochs should we cache child results for
    */
    public AggInitiationSensor(Topology t, int id, Aggregator agg, int dist) {
        super(t,id, agg, dist);

        //create groups, if needed.
        level = 0;
    }

    public AggInitiationSensor(Topology t, int id, Aggregator agg,Aggregator agg1,int dist) {
      super(t,id,agg,agg1,dist);

      //create groups, if needed.
      level = 0;
    }


    public void setTime(int time) {
        super.setTime(time);

        //just send out request once...
        if (!once) {


            Vector aggs = new Vector();
            Object emptyAgg = agg.emptyAgg(this);
            AggMessage am = null;
            aggs.addElement(emptyAgg);

            try {
                am = new AggMessage(this, Clock.getEpoch(time), aggs, 0, null, null, null, agg.numBytes(emptyAgg));
                am.queryID = this.queryID;
                //Random r = new Random(am.queryID);
                //am.sketchInsertKey = r.nextInt();
            } catch (Exception e) {
                //shouldn't happen...
            }
            // ICK -- this is a broken way to generate a hypothesis -- the aggregate function
            // is responsible for doing it without any knowledge from the other sensors....
            if (guess) {
                am.guessAgg = agg.makeGuess();
            } {
                am.guessAgg = null;
            }
            am.setBroadcast(true);
            top.radiateMessage(am, getX(), getY());
            once = true;
        }
    }
}
